@@ -1,20 +1,22 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 ### BEGIN LICENSE
 # Copyright (C) 2011 Didier Roche <didrocks@ubuntu.com>
-# This program is free software: you can redistribute it and/or modify it 
-# under the terms of the GNU General Public License version 3, as published 
+# This program is free software: you can redistribute it and/or modify it
+# under the terms of the GNU General Public License version 3, as published
 # by the Free Software Foundation.
-# 
-# This program is distributed in the hope that it will be useful, but 
-# WITHOUT ANY WARRANTY; without even the implied warranties of 
-# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranties of
+# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 # PURPOSE.  See the GNU General Public License for more details.
-# 
-# You should have received a copy of the GNU General Public License along 
+#
+# You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 ### END LICENSE
 
+import atexit
+import errno
 import json
 import os
 import shutil
@@ -23,9 +25,30 @@ import subprocess
 import time
 import unittest
 
+# For Python 2, because builtin-open has no 'encoding' argument.
+import codecs
+
 sys.path.insert(0, os.path.abspath('.'))
 
-shutil.copy(os.path.join(os.path.dirname(__file__), "data", "oneconf.override"), "/tmp/oneconf.override")
+# Create the override file, but ensure that it gets cleaned up when this test
+# exits.  Because of the way oneconf.paths operates, this file must exist
+# before the import.
+def cleanup():
+    try:
+        os.remove('/tmp/oneconf.override')
+    except OSError as error:
+        if error.errno != errno.ENOENT:
+            raise
+    try:
+        shutil.rmtree('/tmp/oneconf-test')
+    except OSError as error:
+        if error.errno != errno.ENOENT:
+            raise
+atexit.register(cleanup)
+shutil.copy(
+    os.path.join(os.path.dirname(__file__), "data", "oneconf.override"),
+    '/tmp/oneconf.override')
+
 from oneconf import paths
 from oneconf.networksync.fake_webcatalog_silo import FakeWebCatalogSilo
 
@@ -33,7 +56,7 @@ class OneConfSyncing(unittest.TestCase):
 
     def setUp(self):
         os.environ['PYTHONPATH'] = '.:' + os.environ.get('PYTHONPATH', '')
-        self.cmd_line = ["python", "oneconf/networksync/__init__.py"]
+        self.cmd_line = [sys.executable, '-m', 'oneconf.networksync']
         self.hostid = "0000"
         self.hostname = "foomachine"
         self.output = None
@@ -44,7 +67,7 @@ class OneConfSyncing(unittest.TestCase):
         self.src_hostdir = None
 
     def tearDown(self):
-        for key in os.environ.keys():
+        for key in list(os.environ.keys()):
             if "ONECONF_" in key:
                 os.environ.pop(key)
         try:
@@ -57,21 +80,22 @@ class OneConfSyncing(unittest.TestCase):
         output = []
         while True:
             additional_output = process.stderr.readline()
-            print additional_output
+            print(additional_output)
             if not additional_output:
                 break
             output.append(additional_output)
         return output
 
     def msg_in_output(self, output, msg):
-        found = False
         for line in output:
-            found = found or msg in line
-        return found
+            if msg in line:
+                return True
+        return False
 
     def get_daemon_output(self):
         '''Return the daemon output and ensure it's stopped'''
-        p = subprocess.Popen(self.cmd_line, stderr=subprocess.PIPE)      
+        p = subprocess.Popen(self.cmd_line, stderr=subprocess.PIPE,
+                             universal_newlines=True)
         output = self.collect_debug_output(p)
         p.wait()
         p = None
@@ -85,39 +109,74 @@ class OneConfSyncing(unittest.TestCase):
         self.assertFalse(self.msg_in_output(self.output, 'Traceback'))
         if check_errors:
             self.assertFalse(self.msg_in_output(self.output, 'ERROR:'))
-        return (self.msg_in_output(self.output, msg))
+        return self.msg_in_output(self.output, msg)
 
     def copy_state(self, test_ident):
         '''Set state from the test identifier.'''
-        datadir = os.path.join(os.path.dirname(__file__), "data", "syncdatatests")
+        datadir = os.path.join(
+            os.path.dirname(__file__), "data", "syncdatatests")
         self.src_hostdir = os.path.join(datadir, 'host_%s' % test_ident)
-        self.result_hostdir = os.path.join(datadir, 'resulthost_%s' % test_ident)
+        self.result_hostdir = os.path.join(
+            datadir, 'resulthost_%s' % test_ident)
         shutil.copytree(self.src_hostdir, self.hostdir)
         if not os.path.isdir(paths.WEBCATALOG_SILO_DIR):
             os.makedirs(paths.WEBCATALOG_SILO_DIR)
         try:
-            shutil.copy(os.path.join(datadir, 'silo_%s' % test_ident), paths.WEBCATALOG_SILO_SOURCE)
-        except IOError:
-            pass # some tests have no silo source file
+            shutil.copy(os.path.join(datadir, 'silo_%s' % test_ident),
+                        paths.WEBCATALOG_SILO_SOURCE)
+        # For Python 2 - Python 3 could use FileNotFoundError
+        except (IOError, OSError) as error:
+            if error.errno != errno.ENOENT:
+                pass # some tests have no silo source file
 
-    def compare_silo_results(self, hosts_medata, packages_metadata):
-        '''Return True if start and result silos contains identical hosts and pkg'''
+    def compare_silo_results(self, hosts_metadata, packages_metadata):
+        """Return True if start and result silos contains identical hosts
+        and pkg"""
         fakecatalog = FakeWebCatalogSilo(paths.WEBCATALOG_SILO_RESULT)
-        self.assertEqual(fakecatalog._FAKE_SETTINGS['hosts_metadata'], hosts_medata)
-        self.assertEqual(fakecatalog._FAKE_SETTINGS['packages_metadata'], packages_metadata)
+        self.assertEqual(fakecatalog._FAKE_SETTINGS['hosts_metadata'],
+                         hosts_metadata)
+        self.assertEqual(fakecatalog._FAKE_SETTINGS['packages_metadata'],
+                         packages_metadata)
 
     def compare_files(self, file1, file2):
         '''Compare file content'''
-        src_content = open(file1).read().splitlines()
-        dest_content = open(file2).read().splitlines()
-        self.assertEqual(src_content, dest_content)
+        # We have to be moderately intellegent in comparing these files.  Some
+        # of them are json serialized dictionaries, so straight textual
+        # comparisons won't work (dictionary order is not predictable).
+        # Unfortunately, there's no way to know whether we're looking at json
+        # or something else.  So, first try to deserialize the content and if
+        # that fails, use the string representations.
+        #
+        # Yes, we have to use a backslash here. :\
+        try:
+            with codecs.open(file1, 'r', encoding='utf-8') as fp1, \
+                 codecs.open(file2, 'r', encoding='utf-8') as fp2:
+                src_string = fp1.read()
+                dst_string = fp2.read()
+        except UnicodeDecodeError:
+            # If it wasn't even utf-8, then it isn't json, so re-open the
+            # files as binary and compare the byte strings.
+            with open(file1, 'rb') as fp1, open(file2, 'rb') as fp2:
+                src_content = fp1.read()
+                dst_content = fp2.read()
+            self.assertEqual(src_content, dst_content)
+            return
+        try:
+            src_content = json.loads(src_string)
+            dst_content = json.loads(dst_string)
+        except TypeError:
+            # Again, it's not json.
+            self.assertMultiLineEqual(src_string, dst_string)
+        else:
+            self.assertEqual(src_content, dst_content)
 
     def compare_dirs(self, source, dest):
         '''Compare directory files, ignoring the last_sync file on purpose'''
         for filename in os.listdir(source):
             if filename == paths.LAST_SYNC_DATE_FILENAME:
                 continue
-            self.compare_files(os.path.join(source, filename), os.path.join(dest, filename))
+            self.compare_files(os.path.join(source, filename),
+                               os.path.join(dest, filename))
 
     def test_no_sync_no_network(self):
         '''Test that no sync is happening if no network'''
@@ -136,7 +195,8 @@ class OneConfSyncing(unittest.TestCase):
     def test_first_sync(self):
         '''Test a first synchronisation without any data on the webcatalog'''
         self.copy_state('nosilo_nopackage_onlyhost')
-        self.assertTrue(self.check_msg_in_output("Push current host to infra now"))
+        self.assertTrue(
+            self.check_msg_in_output("Push current host to infra now"))
         self.assertTrue(self.check_msg_in_output("New host registered done"))
         self.assertFalse(self.check_msg_in_output("emit_new_hostlist"))
         self.assertFalse(self.check_msg_in_output("emit_new_packagelist"))
@@ -145,7 +205,8 @@ class OneConfSyncing(unittest.TestCase):
                                                  'logo_checksum': None,
                                                  'packages_checksum': None}},
                                   {})
-        self.compare_dirs(self.src_hostdir, self.hostdir) # Ensure nothing changed in the source dir
+        # Ensure nothing changed in the source dir
+        self.compare_dirs(self.src_hostdir, self.hostdir)
 
     def test_date_synchro(self):
         '''Ensure a synchro date is written, older than current time, and right signal emitted'''
@@ -155,7 +216,8 @@ class OneConfSyncing(unittest.TestCase):
         self.assertTrue(self.check_msg_in_output("Saving updated %s to disk" % sync_file))
         self.assertTrue(self.check_msg_in_output("emit_new_lastestsync"))
         with open(sync_file, 'r') as f:
-            current_host = self.assertTrue(json.load(f)["last_sync"] > now)
+            contents = json.load(f)
+        self.assertGreater(float(contents['last_sync']), now)
 
     def test_host_not_shared(self):
         '''Test a non shared host is really not shared'''
@@ -192,7 +254,7 @@ class OneConfSyncing(unittest.TestCase):
         self.assertTrue(self.check_msg_in_output("Ensure that current host is not shared"))
         self.assertFalse(self.check_msg_in_output("Can't delete current host from infra: Host Not Found"))
         self.compare_silo_results({}, {})
-        
+
     def test_unshare_other_host(self):
         '''Unshare a host which is not the current one'''
         self.copy_state('unshare_other_host')
@@ -227,7 +289,7 @@ class OneConfSyncing(unittest.TestCase):
                                   {u'0000': {u'bar': {u'auto': True},
                                              u'baz': {u'auto': False},
                                              u'foo': {u'auto': False}}})
-        
+
 
     def test_update_host_with_hostname_change(self):
         '''Update a host only changing the hostname'''
@@ -246,14 +308,17 @@ class OneConfSyncing(unittest.TestCase):
     def test_update_packages_for_host(self):
         '''Update a package list for current host'''
         self.copy_state('update_packages_for_current_host')
-        self.assertTrue(self.check_msg_in_output("Check if packages for current host need to be pushed to infra"))
+        self.assertTrue(self.check_msg_in_output(
+            "Check if packages for current host need to be pushed to infra"))
         self.assertTrue(self.check_msg_in_output("Push new packages"))
-        self.compare_silo_results({self.hostid: {'hostname': self.hostname,
-                                                 'logo_checksum': None,
-                                                 'packages_checksum': u'AAAA'}},
-                                  {self.hostid: {u'fol': {u'auto': False},
-                                                 u'bar': {u'auto': True},
-                                                 u'baz': {u'auto': True}}})
+        self.compare_silo_results(
+            {self.hostid: {'hostname': self.hostname,
+                           'logo_checksum': None,
+                           'packages_checksum': u'AAAA'}},
+            {self.hostid: {u'fol': {u'auto': False},
+                           u'bar': {u'auto': True},
+                           u'baz': {u'auto': True}}}
+            )
 
     def test_get_firsttime_sync_other_host(self):
         '''First time getting another host, no package'''
@@ -261,7 +326,7 @@ class OneConfSyncing(unittest.TestCase):
         self.assertTrue(self.check_msg_in_output("Refresh new host"))
         self.assertTrue(self.check_msg_in_output("Saving updated /tmp/oneconf-test/cache/0000/other_hosts to disk"))
         self.assertFalse(self.check_msg_in_output("Refresh new packages"))
-        self.assertFalse(self.check_msg_in_output("Refresh new logo"))        
+        self.assertFalse(self.check_msg_in_output("Refresh new logo"))
         self.assertTrue(self.check_msg_in_output("emit_new_hostlist not bound to anything"))
         self.compare_dirs(self.result_hostdir, self.hostdir)
 
@@ -274,9 +339,9 @@ class OneConfSyncing(unittest.TestCase):
         self.assertTrue(self.check_msg_in_output("Saving updated /tmp/oneconf-test/cache/0000/other_hosts to disk"))
         self.assertTrue(self.check_msg_in_output("emit_new_hostlist not bound to anything"))
         self.assertTrue(self.check_msg_in_output("emit_new_packagelist(AAAA) not bound to anything"))
-        self.compare_dirs(self.result_hostdir, self.hostdir)        
-        
-    def test_sync_other_host_with_updated_packages(self):    
+        self.compare_dirs(self.result_hostdir, self.hostdir)
+
+    def test_sync_other_host_with_updated_packages(self):
         '''Sync another host with updated packages'''
         self.copy_state('sync_other_host_with_updated_packages')
         self.assertTrue(self.check_msg_in_output("Refresh new packages"))
@@ -295,7 +360,7 @@ class OneConfSyncing(unittest.TestCase):
         self.assertTrue(self.check_msg_in_output("Saving updated /tmp/oneconf-test/cache/0000/other_hosts to disk"))
         self.assertTrue(self.check_msg_in_output("emit_new_hostlist not bound to anything"))
         self.compare_dirs(self.result_hostdir, self.hostdir)
-    
+
     def test_sync_a_newhost_with_already_other_hosts(self):
         '''Add an additional host with some already there'''
         self.copy_state('sync_a_newhost_with_already_other_hosts')
@@ -306,16 +371,18 @@ class OneConfSyncing(unittest.TestCase):
         self.assertTrue(self.check_msg_in_output("Saving updated /tmp/oneconf-test/cache/0000/other_hosts to disk"))
         self.assertTrue(self.check_msg_in_output("emit_new_hostlist not bound to anything"))
         self.compare_dirs(self.result_hostdir, self.hostdir)
-        
+
     def test_sync_remove_other_host(self):
         '''Remove a host after a sync'''
         self.copy_state('sync_remove_other_host')
         self.assertTrue(self.check_msg_in_output("Refresh new host"))
-        self.assertTrue(self.check_msg_in_output("Saving updated /tmp/oneconf-test/cache/0000/other_hosts to disk"))
-        self.assertTrue(self.check_msg_in_output("emit_new_hostlist not bound to anything"))
+        self.assertTrue(self.check_msg_in_output(
+            "Saving updated /tmp/oneconf-test/cache/0000/other_hosts to disk"))
+        self.assertTrue(self.check_msg_in_output(
+            "emit_new_hostlist not bound to anything"))
         self.assertFalse(self.check_msg_in_output("emit_new_packagelist"))
         self.compare_dirs(self.result_hostdir, self.hostdir)
-        
+
     def test_server_error(self):
         '''Test server not responsing at all'''
         self.copy_state('fake_server_errors')
@@ -352,17 +419,17 @@ class OneConfSyncing(unittest.TestCase):
     def test_get_packages_error(self):
         '''Test when getting all packages errors'''
         self.copy_state('fake_server_errors')
-        os.environ["ONECONF_list_packages_error"] = "True"      
+        os.environ["ONECONF_list_packages_error"] = "True"
         self.assertTrue(self.check_msg_in_output("Invalid package data from server: Fake WebCatalogAPI raising fake exception", check_errors=False))
         self.assertTrue(self.check_msg_in_output("Saving updated", check_errors=False))
-        
+
     def test_delete_current_host_error(self):
         '''Try to delete the current host and there is an error'''
         self.copy_state('delete_current_host_error')
         os.environ["ONECONF_delete_machine_error"] = "True"
         self.assertTrue(self.check_msg_in_output("Can't delete current host from infra: Fake WebCatalogAPI raising fake exception", check_errors=False))
         self.assertTrue(self.check_msg_in_output("Saving updated", check_errors=False))
-        
+
     def test_update_current_host_error(self):
         '''Update an already registered current host and there is an error'''
         self.copy_state('fake_server_errors')
@@ -370,7 +437,7 @@ class OneConfSyncing(unittest.TestCase):
         self.assertTrue(self.check_msg_in_output("Can't update machine: Fake WebCatalogAPI raising fake exception", check_errors=False))
         self.assertFalse(self.check_msg_in_output("Host data refreshed", check_errors=False))
         self.assertTrue(self.check_msg_in_output("Saving updated", check_errors=False))
-        
+
     def test_create_current_host_error(self):
         '''Try to create the current host and there is an error'''
         self.copy_state('nosilo_nopackage_onlyhost')
@@ -378,22 +445,22 @@ class OneConfSyncing(unittest.TestCase):
         self.assertTrue(self.check_msg_in_output("Can't register new host: Fake WebCatalogAPI raising fake exception", check_errors=False))
         self.assertFalse(self.check_msg_in_output("New host registered done", check_errors=False))
         self.assertTrue(self.check_msg_in_output("Saving updated", check_errors=False))
-        
+
     def test_update_package_list_error(self):
         '''Try to update the remove package list and get an error'''
         self.copy_state('fake_server_errors')
         os.environ["ONECONF_update_packages_error"] = "True"
         self.assertTrue(self.check_msg_in_output("Can't push current package list: Fake WebCatalogAPI raising fake exception", check_errors=False))
         self.assertTrue(self.check_msg_in_output("Saving updated", check_errors=False))
-        
+
     def test_sync_with_broken_pending_file(self):
         '''Try to update with a broken pending file, should just ignore it'''
         self.copy_state('broken_pending_file')
         self.check_msg_in_output("The pending file is broken, ignoring", check_errors=False)
-        
+
     def test_no_sync_with_invalid_setup(self):
         '''Test that no sync and no traceback is happening if we have an invalid setup'''
-        self.cmd_line = ["python", "oneconf/networksync/__init__.py", "--no-infra-client"]
+        self.cmd_line.append('--no-infra-client')
         shutil.copy(os.path.join(os.path.dirname(__file__), "data", "oneconf.invaliddistro.override"), "/tmp/oneconf.override")
         self.assertFalse(self.check_msg_in_output("Start processing sync"))
 
@@ -401,10 +468,10 @@ class OneConfSyncing(unittest.TestCase):
 # main
 #
 if __name__ == '__main__':
-    print '''
+    print('''
     #########################################
     #         Test OneConf syncing          #
     #########################################
-    '''
+    ''')
     unittest.main(exit=False)
     os.remove("/tmp/oneconf.override")
